@@ -1,6 +1,6 @@
 #' NNS Boost
 #'
-#' Ensemble method for classification using the predictions of the NNS multivariate regression \link{NNS.reg} collected from uncorrelated feature combinations.
+#' Ensemble method for classification using the NNS multivariate regression \link{NNS.reg} as the base learner instead of trees.
 #'
 #' @param IVs.train a matrix or data frame of variables of numeric or factor data types.
 #' @param DV.train a numeric or factor vector with compatible dimensions to \code{(IVs.train)}.
@@ -62,21 +62,21 @@ NNS.boost <- function(IVs.train,
                       features.only = FALSE,
                       feature.importance = TRUE,
                       status = TRUE){
-
+  
   if(sum(is.na(cbind(IVs.train,DV.train))) > 0) stop("You have some missing values, please address.")
-
+  
   if(is.null(obj.fn)) stop("Please provide an objective function")
-
+  
   if(balance && is.null(type)) warning("type = 'CLASS' selected due to balance = TRUE.")
   if(balance) type <- "CLASS"
-
-
+  
+  
   if(!is.null(type) && min(as.numeric(as.factor(DV.train)))==0) warning("Base response variable category should be 1, not 0.")
-
-
+  
+  
   if(any(class(IVs.train)%in%c("tbl","data.table"))) IVs.train <- as.data.frame(IVs.train)
   if(any(class(DV.train)%in%c("tbl","data.table"))) DV.train <- as.vector(unlist(DV.train))
-
+  
   if(!is.null(type)){
     type <- tolower(type)
     if(type == "class" && identical(obj.fn,expression( sum((predicted - actual)^2) ))){
@@ -84,157 +84,171 @@ NNS.boost <- function(IVs.train,
       objective <- "max"
     }
   }
-
+  
   objective <- tolower(objective)
-
+  
   if(is.null(colnames(IVs.train))){
     colnames.list <- lapply(1 : dim(IVs.train)[2], function(i) paste0("X", i))
     colnames(IVs.test) <- colnames(IVs.train) <- as.character(colnames.list)
   }
-
+  
   features <- colnames(IVs.train)
+  IVs.train <- IVs.train[ ,sort(features)]
 
   transform <- data.matrix(cbind(DV.train, IVs.train))
-
+  
   IVs.train <- transform[,-1]
-  colnames(IVs.train) <- features
-
+  colnames(IVs.train) <- sort(features)
+  
   DV.train <- transform[,1]
-
-  original.par <- par(no.readonly = TRUE)
-
+  
+  
+  
   if(is.null(IVs.test)){
     IVs.test <- IVs.train
   } else {
     if(any(class(IVs.test)%in%c("tbl","data.table"))) IVs.test <- as.data.frame(IVs.test)
   }
-
-
-
+  
+  
+  
   if(balance){
     DV.train <- as.numeric(as.factor(DV.train))
-
+    
     y_train <- as.factor(as.character(DV.train))
-
+    
     training_1 <- do.call(cbind, caret::downSample(IVs.train, y_train, list = TRUE))
     training_2 <- do.call(cbind, caret::upSample(IVs.train, y_train, list = TRUE))
-
+    
     training <- rbind.data.frame(training_1, training_2)
-
+    
     colnames(training) <- c(colnames(IVs.train), names(DV.train))
-
+    
     IVs.train <- training[, -ncol(training)]
     DV.train <- as.numeric(as.character(training[,ncol(training)]))
   }
-
-
-
+  
+  
+  
   x <- data.table::data.table(IVs.train)
   y <- DV.train
   z <- IVs.test
-
-
+  
+  
   ### Representative samples
-      rep.x <- x[,lapply(.SD, function(z) fivenum(as.numeric(z))), by = .(y)]
-
-      rep.y <- unlist(rep.x[,1])
-      rep.x <- rep.x[,-1]
-
-      rep.x <- as.data.frame(rep.x)
-
-
+  yx <- cbind(y, x)
+  rep.x <- yx[,lapply(.SD, function(z) fivenum(as.numeric(z)))]
+  rm(yx)
+  
+  rep.y <- unlist(rep.x[,1])
+  rep.x <- rep.x[,-1]
+  
+  rep.x <- as.data.frame(rep.x)
+  
+  
   n <- ncol(x)
-
+  
   if(is.null(epochs)) epochs <- 2*length(y)
-
+  
   if(!is.null(ts.test)) dist <- "DTW" else dist <- "L2"
-
+  
   estimates <- list()
   fold <- list()
-
+  
   old.threshold <- 0
-
+  
+  sets <- sum(choose(n, 1:n))
+  deterministic <- FALSE 
+  if((sets < length(y)) || n <= 10){
+    deterministic <- TRUE
+    learner.trials <- sets
+    combn_vec <- Vectorize(Rfast::comb_n, vectorize.args = "k")
+    deterministic.sets <- unlist(lapply(combn_vec(n, 1:n), function(df) as.list(as.data.frame(df))), recursive = FALSE)
+  }
+  
   # Add test loop for highest threshold ...
   if(is.null(threshold)){
     if(!extreme) epochs <- NULL
     if(is.null(CV.size)) new.CV.size <- round(runif(1, .2, 1/3), 3) else new.CV.size <- CV.size
-
+    
     old.threshold <- 1
-
+    
     if(is.null(learner.trials)){learner.trials <- length(y)}
-
-    learner.trials <- min(sum(choose(n, 1:n)), learner.trials)
-
+    
     results <- numeric(learner.trials)
     test.features <- vector(mode = "list", learner.trials)
-
+    
     for(i in 1:learner.trials){
       set.seed(123 + i)
-
+      
       l <- length(y)
       if(i<=l/4) new.index <- as.integer(seq(i, length(y), length.out = as.integer(new.CV.size * length(y)))) else {
-          new.index <- sample(l, as.integer(new.CV.size * l), replace = FALSE)
+        new.index <- sample(l, as.integer(new.CV.size * l), replace = FALSE)
       }
-
-
+      
+      
       if(!is.null(ts.test)) new.index <- 1:(length(y) - ts.test)
-
+      
       new.index <- unlist(new.index)
+      
+      new.iv.train <- cbind(y[-new.index], x[-new.index,]) 
+      new.iv.train <- new.iv.train[,lapply(.SD, as.double)]
+      
+      new.iv.train <- new.iv.train[,lapply(.SD, function(z) fivenum(as.numeric(z)))]
+      
+      new.dv.train <- unlist(new.iv.train[,1])
+      new.iv.train <- as.data.frame(new.iv.train)
+      new.iv.train <- new.iv.train[,unlist(colnames(new.iv.train)%in%colnames(IVs.train))]
 
-          new.iv.train <- x[-new.index,] 
-          new.iv.train <- new.iv.train[,lapply(.SD, as.double)]
-
-          new.iv.train <- new.iv.train[,lapply(.SD,function(z) fivenum(as.numeric(z))), by = .(y[-new.index])]
-
-          new.dv.train <- unlist(new.iv.train[,1])
-          new.iv.train <- as.data.frame(new.iv.train[,-1])
-
-          new.iv.train <- rbind(new.iv.train, data.matrix(x[-new.index,]))
-          new.dv.train <- c(new.dv.train, y[-new.index])
-
-
-      colnames(new.iv.train) <- features
-
+      
+      new.iv.train <- data.table::rbindlist(list(new.iv.train, x[-new.index,]), use.names = FALSE)
+      new.dv.train <- c(new.dv.train, y[-new.index])
+      
+      colnames(new.iv.train) <- c( features)
+      
       actual <- as.numeric(y[new.index])
       new.iv.test <- x[new.index,]
-
-      if(status){
-        message("Current Threshold Iterations Remaining = " ,learner.trials+1-i," ","\r",appendLF=FALSE)
-      }
-
-      test.features[[i]] <- sort(sample(n, sample(2:(n-1), 1), replace = FALSE))
-
+      
+      if(status) message("Current Threshold Iterations Remaining = " ,learner.trials+1-i," ","\r",appendLF=FALSE)
+      
+      if(deterministic) test.features[[i]] <- deterministic.sets[i] else test.features[[i]] <- sort(sample(n, sample(2:n, 1), replace = FALSE))
+      
+      if(length(test.features[[i]]) == 1) point.est.values <- unlist(new.iv.test[,test.features[[i]]]) else point.est.values <- new.iv.test[,test.features[[i]]]
+      
+      learning.IVs <- new.iv.train[,unlist(test.features[i])]
+     
       #If estimate is > threshold, store 'features'
-      predicted <- NNS.reg(new.iv.train[,test.features[[i]]],
+      predicted <- NNS.reg(learning.IVs,
                            new.dv.train,
-                           point.est = new.iv.test[,test.features[[i]]],
+                           point.est = point.est.values,
                            dim.red.method = "equal",
                            plot = FALSE, order = depth,
                            ncores = 1, type = type)$Point.est
-
+      
       predicted[is.na(predicted)] <- gravity(na.omit(predicted))
-
+      
       # Do not predict a new unseen class
       if(!is.null(type)){
         predicted <- pmin(predicted, max(as.numeric(y)))
         predicted <- pmax(predicted, min(as.numeric(y)))
       }
-
+      
       results[i] <- eval(obj.fn)
-
+      
     } # i in learner.trials
   } else {
     results <- threshold
   } # NULL threshold
-
+  
 
   if(extreme){
     if(objective=="max") threshold <- max(results) else threshold <- min(results)
   } else {
     if(objective=="max") threshold <- fivenum(results)[4] else threshold <- fivenum(results)[2]
   }
-
+  
   if(feature.importance){
+    original.par <- par(no.readonly = TRUE)
     par(mfrow = c(2,1))
     par(mai = c(1.0,.5,0.8,0.5))
     hist(results, main = "Distribution of Learner Trials Objective Function",
@@ -247,34 +261,36 @@ NNS.boost <- function(IVs.train,
       if(objective=='max') mtext("Threshold >", side = 3, col = "red", at = threshold) else mtext("< Threshold", side = 3, col = "red", at = threshold)
     }
   }
-
-
-
+  
+  
+  
   if(status){
     message(paste0("Learner Accuracy Threshold = ", format(threshold, digits = 3, nsmall = 2),"           "), appendLF = TRUE)
-
+    
     # Clear message line
     message("                                       ", "\r", appendLF = FALSE)
   }
-
+  
   if(extreme){
-      if(objective=="max") reduced.test.features <- test.features[which.max(results)] else reduced.test.features <- test.features[which.min(results)]
+    if(objective=="max") reduced.test.features <- test.features[which.max(results)] else reduced.test.features <- test.features[which.min(results)]
   } else {
-      if(objective=="max") reduced.test.features <- test.features[which(results>=threshold)] else reduced.test.features <- test.features[which(results<=threshold)]
+    if(objective=="max") reduced.test.features <- test.features[which(results>=threshold)] else reduced.test.features <- test.features[which(results<=threshold)]
   }
 
   rf <- data.table::data.table(table(as.character(reduced.test.features)))
   rf$N <- rf$N / sum(rf$N)
 
   rf_reduced <- apply(rf, 1, function(x) eval(parse(text=x[1])))
-
+  
   scale_factor_rf <- table(unlist(rf_reduced))/min(table(unlist(rf_reduced)))
-
+    
   reduced.test.features <- as.numeric(rep(names(scale_factor_rf), ifelse(scale_factor_rf%%1 < .5, floor(scale_factor_rf), ceiling(scale_factor_rf))))
-
+  
   keeper.features <- list()
-
-  if(!is.null(epochs)){
+  
+  if(deterministic) epochs <- NULL
+  
+  if(!is.null(epochs) && !deterministic){
     
     if(is.null(CV.size)) new.CV.size <- round(runif(1, .2, 1/3), 3) else new.CV.size <- CV.size
     
@@ -286,53 +302,58 @@ NNS.boost <- function(IVs.train,
         new.index <- sample(l, as.integer(new.CV.size * l), replace = FALSE)
       }
       if(!is.null(ts.test)) new.index <- length(y) - (2*ts.test):0
-
+      
       new.index <- unlist(new.index)
-
-
-          new.iv.train <- x[-new.index, ] #data.table::data.table(x[-new.index, ])
-          new.iv.train <- new.iv.train[, lapply(.SD,as.double)]
-
-          new.iv.train <- new.iv.train[,lapply(.SD,fivenum), by = .(y[-new.index])]
-
-          new.dv.train <- unlist(new.iv.train[, 1])
-          new.iv.train <- as.data.frame(new.iv.train[, -1])
-
-          new.iv.train <- rbind(new.iv.train, data.matrix(x[-new.index,]))
-          new.dv.train <- c(new.dv.train, y[-new.index])
-
-
+      
+      
+      new.iv.train <- cbind(y[-new.index], x[-new.index, ])
+      new.iv.train <- new.iv.train[, lapply(.SD, as.double)]
+      
+      new.iv.train <- new.iv.train[,lapply(.SD,fivenum), by = .(y[-new.index])]
+      noise_idx <- which(colnames(new.iv.train)=="NNS_NOISE")
+            
+      new.dv.train <- unlist(new.iv.train[, 1])
+      new.iv.train <- as.data.frame(new.iv.train)
+      new.iv.train <- new.iv.train[,unlist(colnames(new.iv.train)%in%colnames(IVs.train))]
+      
+      new.iv.train <- data.table::rbindlist(list(new.iv.train, x[-new.index,]), use.names = FALSE)
+      new.dv.train <- c(new.dv.train, y[-new.index])
+      
+      
       actual <- as.numeric(y[new.index])
       new.iv.test <- x[new.index,]
-
+      
       if(status){
         message("% of epochs = ", format(j/epochs,digits =  3,nsmall = 2),"     ","\r",appendLF=FALSE)
-
+        
         if(j == epochs){
           message("% of epochs ",j," = 1.000     ","\r",appendLF = FALSE)
           flush.console()
         }
       }
-
-      features <- sort(c(unlist(reduced.test.features), sample(c(1:n), sample(1:n, 1), replace = FALSE)))
-
+      
+      if(deterministic) features <- unlist(deterministic.sets[[j]]) else features <- sort(c(unlist(reduced.test.features), sample(c(1:n), sample(1:n, 1), replace = FALSE)))
+    
+      if(length(features) == 1) point.est.values <- unlist(new.iv.test[, as.numeric(features)]) else point.est.values <- new.iv.test[, as.numeric(features)]
+      
+      
       #If estimate is > threshold, store 'features'
-      predicted <- NNS.reg(new.iv.train[, features],
-                           new.dv.train, point.est = new.iv.test[, features],
+      predicted <- NNS.reg(new.iv.train[, as.numeric(features)],
+                           new.dv.train, point.est = point.est.values,
                            dim.red.method = "equal",
                            plot = FALSE, residual.plot = FALSE, order = depth,
                            ncores = 1, type = type)$Point.est
-
+      
       predicted[is.na(predicted)] <- gravity(na.omit(predicted))
       # Do not predict a new unseen class
       if(!is.null(type)){
         predicted <- pmin(predicted,max(as.numeric(y)))
         predicted <- pmax(predicted,min(as.numeric(y)))
       }
-
-
+      
+      
       new.results <- eval(obj.fn)
-
+      
       if(objective=="max"){
         if(is.na(new.results)) new.results <- .99*threshold
         if(new.results>=threshold) keeper.features[[j]] <- features else keeper.features[[j]] <- NULL
@@ -342,9 +363,9 @@ NNS.boost <- function(IVs.train,
       }
     }
   } else { # !is.null(epochs)
-      keeper.features <- reduced.test.features
+    keeper.features <- reduced.test.features
   }
-
+  
   keeper.features <- keeper.features[!sapply(keeper.features, is.null)]
   if(length(keeper.features)==0){
     if(old.threshold==0){
@@ -355,57 +376,55 @@ NNS.boost <- function(IVs.train,
   }
 
   plot.table <- table(unlist(keeper.features))
-
+     
   names(plot.table) <- colnames(IVs.train)[eval(as.numeric(names(plot.table)))]
-
+  
   if(features.only || feature.importance) plot.table <- plot.table[rev(order(plot.table))]
 
-
+  
   if(features.only){
-      par(mfrow=c(1,1))
-      par(original.par)
-      return(list("feature.weights" = plot.table/sum(plot.table),
-                  "feature.frequency" = plot.table))
+    return(list("feature.weights" = plot.table/sum(plot.table),
+                "feature.frequency" = plot.table))
   }
-
-
+  
+  
   if(!is.null(rep.y)){
-      x <- rbind(rep.x, (x))
-      y <- c(rep.y, y)
+    x <- rbind(rep.x, (x))
+    y <- c(rep.y, y)
   }
-
+  
   kf <- data.table::data.table(table(as.character(keeper.features)))
   kf$N <- kf$N / sum(kf$N)
-
+  
   kf_reduced <- apply(kf, 1, function(x) eval(parse(text=x[1])))
-
+  
   scale_factor <- table(unlist(kf_reduced))/min(table(unlist(kf_reduced)))
-
+  
   final_scale <- as.numeric(rep(names(scale_factor), ifelse(scale_factor%%1 < .5, floor(scale_factor), ceiling(scale_factor))))
-
+  
   if(status) message("Generating Final Estimate" ,"\r", appendLF = TRUE)
-
-      estimates <- NNS.stack(x[, unlist(final_scale)],
-                             y,
-                             IVs.test = z[, unlist(final_scale)],
-                             order = depth, dim.red.method = "all",
-                             ncores = 1,
-                             stack = FALSE, status = status,
-                             type = type, dist = dist, folds = folds)$stack
-
-      estimates[is.na(unlist(estimates))] <- ifelse(!is.null(type), mode_class(unlist(na.omit(estimates))), mode(unlist(na.omit(estimates))))
-
-
+  
+  estimates <- NNS.stack(x[, unlist(final_scale)],
+                         y,
+                         IVs.test = z[, unlist(final_scale)],
+                         order = depth, dim.red.method = "all",
+                         ncores = 1,
+                         stack = FALSE, status = status,
+                         type = type, dist = dist, folds = folds)$stack
+  
+  estimates[is.na(unlist(estimates))] <- ifelse(!is.null(type), mode_class(unlist(na.omit(estimates))), mode(unlist(na.omit(estimates))))
+  
+  
   if(!is.null(type)){
     estimates <- pmin(estimates, max(as.numeric(y)))
     estimates <- pmax(estimates, min(as.numeric(y)))
   }
-
-
+  
+  
   if(feature.importance){
     linch <-  max(strwidth(names(plot.table), "inch") + 0.4, na.rm = TRUE)
     par(mai=c(1.0, linch, 0.8, 0.5))
-
+    
     if(length(plot.table)!=1){
       barplot(sort(plot.table, decreasing = FALSE)[1:min(n, 10)],
               horiz = TRUE,
@@ -422,12 +441,12 @@ NNS.boost <- function(IVs.train,
     par(mfrow=c(1,1))
     par(original.par)
   }
-
-
+  
+  
   if(!is.null(type)) estimates <- ifelse(estimates%%1 < .5, floor(estimates), ceiling(estimates))
-
+  
   return(list("results" = estimates,
               "feature.weights" = plot.table/sum(plot.table),
               "feature.frequency" = plot.table))
-
+  
 }
