@@ -26,7 +26,8 @@
 #' dy.dx(x, y, eval.point = 1.75)
 #'
 #' # Vector of derivatives
-#' dy.dx(x, y, eval.point = c(1.75, 2.5), deriv.method = "FD")}
+#' dy.dx(x, y, eval.point = c(1.75, 2.5), deriv.method = "FD")
+#' }
 #' @export
 
 dy.dx <- function(x, y, eval.point = median(x), deriv.method = "FD"){
@@ -38,7 +39,7 @@ dy.dx <- function(x, y, eval.point = median(x), deriv.method = "FD"){
 
   order <- NULL
 
-  dep <- NNS.dep(x, y, asym = TRUE, ncores = 1)$Dependence
+  dep <- NNS.dep(x, y, asym = TRUE)$Dependence
 
   h <- mean(abs(diff(LPM.VaR(seq(0, 1, min(.05, max(.01, 1-dep))), 1, x))))
 
